;;; Example use of JDBC.
;;; For this to work with Oracle you need Oracle's jdbcthin\lib\classes111.zip
;;; in your class path.  ODBC support is built in to JDK 1.2.
(import "java.math.BigDecimal")
(import "java.sql.DriverManager")

(define (map-columns f row result n)
  ;; Map (f row n) over the n columns of row putting the result in 
  ;; the vector result.
  (if (= n 0) result
      (let ((i (- n 1)))
	(vector-set! result i (f row n))
	   (map-columns f row result i))))

(define (get-datum row n)
  ;; Get a cell value.
  (let ((it (.getObject row n)))
    (if (eq? it #null) it
	(datum-value it))))

(define-method (datum-value (value java.lang.Object)) value)
;;; KRA 03MAR00: Strings are often duplicates, so intern them all.
(define-method (datum-value (value String)) (.intern value))
(define-method (datum-value (value java.math.BigDecimal))
  ((if (= (.scale value) 0)
       .longValue
       .doubleValue)
   value))

(define (collect-query con query)
  ;; Collect a list of vectors representing the results of a query.
  (let* ((stmt (.createStatement con))
	 (result (.executeQuery stmt query))
	 (brick  (result-brick result)))
    (.close stmt)
    brick))
    
(define (result-brick result)
  ;; Convert a result set into standard form.
  (let* ((meta (.getMetaData result))
	 (ncol (.getColumnCount meta))
	 (col (map-columns (lambda (row n) (.getColumnName row n))
			   meta (make-vector ncol) ncol))
	 (data '()))
    (iterate
     result
     (lambda (result)
       (set! data (cons (map-columns get-datum result (make-vector ncol) ncol)
			data))))
    (cons col (reverse data))))

(define-method (iterate (result java.sql.ResultSet) f)
  (if (.next result)
      (begin (f result)
	     (iterate result f))))

;;; Example:
;;; Register the driver. 
(define (register-driver type)
  (DriverManager.registerDriver
   ((constructor
     (cond
      ((eq? type 'oracle) "oracle.jdbc.driver.OracleDriver") ; Needs a jar.
      ((eq? type 'odbc) "sun.jdbc.odbc.JdbcOdbcDriver")))))) ; Comes with JDK.

;;; Connect to Stout's database
'(define dart-con (DriverManager.getConnection 
	     "jdbc:oracle:thin:@stout:1521:dart" "ntranscom" "ntranscom"))

;;; Sample queries.
'(define countries (collect-query dart-con "select unique(country_state_code), country_state_long_name from geoloc"))

'(define con (DriverManager.getConnection
	      "jdbc:oracle:thin:@kingdom:1521:dart8i" "icis" "icis99"))

'(define con 
  (DriverManager.getConnection
   "jdbc:oracle:thin:@horse:1521:dart" "ntranscom" "ntranscom"))

'(define con 
  (DriverManager.getConnection
   "jdbc:oracle:thin:@oberon.tds-gn.lmco.com:1521:jlactddm" "bbnteam" "bbnteam"))


(define (describe-table con table-name)
  ;; Describe a table.  may not be portable, but works in Oracle.
  (collect-query 
   con 
   (print (string-append
    "select column_name, Data_Type, Data_length, Data_scale from all_Tab_columns where table_name = '" table-name "'"))))
